# -*- coding: utf-8 -*-

# loaders.py
# This file is part of Qarte
#    
# Author : Vincent Vande Vyvre <vincent.vandevyvre@swing.be>
# Copyright: 2011-2013 Vincent Vande Vyvre
# Some parts of code were found in
# arte+7recorder and TVDownloader
# See </usr/share/doc/qarte/copyright>
# Licence: GPL3
# Home page : https://launchpad.net/qarte
#
# Movie's loaders for arte+7 and arteLiveWeb

import sys
import os
import time
import re
import subprocess
import select
import datetime
import shutil
import gettext
import logging
logger = logging.getLogger("loader")

from PyQt4.QtCore import QObject, pyqtSignal, QString, QLocale
from PyQt4.QtGui import QApplication

# http://www.doughellmann.com/PyMOTW/select/

class PlusLoader(QObject):
    loadProgress = pyqtSignal(float)
    loadingFailed = pyqtSignal(str, str)
    loadingComplete = pyqtSignal()
    def __init__(self, main):
        # Parts of this code are come from arte+7Recorder
        # https://launchpad.net/arte+7recorder
        super(PlusLoader, self).__init__()
        self.main = main
        self.lang = main.lang
        self.is_loading = False
        self.kill_request = False
        self.url = None
        self.target = None
        self.pid = None
        self.is_daemon = False

    def set_url(self, url):
        self.url = url

    def set_target(self, target):
        if self.is_daemon:
            # When python is invoked by cron the file system encoding
            # is not usable. Maybe related with this issue:
            # http://bugs.python.org/issue13643
            self.target = target.encode('utf-8', 'replace')
        else:
            self.target = target.encode(sys.getfilesystemencoding(), 'replace')

    def set_destination(self, dest):
        if self.is_daemon:
            self.dest = dest.encode('utf-8', 'replace')

        else:
            self.dest = dest.encode(sys.getfilesystemencoding(), 'replace')

    def load(self):
        try:
            for percent in self.rtmp_download(self.url, self.target):

                if percent == -1.0:
                    raise IOError()

                if percent == 100:
                    self.rename_target()
                    self.loadingComplete.emit()

                self.loadProgress.emit(percent)
                if self.main.abort_download or self.main.stop:
                    self.abort_subprocess()
                    break

        except IOError as why:
            if self.is_daemon:
                self.print_(u"Downloading error : {0}".format(why))

            self.abort_subprocess()
            self.loadingFailed.emit('Error', str(why))
            self.remove_tempfile()


    def rtmp_download(self, link, destination="/dev/null", try_resume=True, 
                                                            resuming=False):
        some_dl_done = False
        need_more_dl = True
        if try_resume and os.path.isfile(destination):
            for percent in self.rtmp_download(link, destination, False, True):
                if percent != -1:
                    some_dl_done = True
                    need_more_dl = percent != 100.0
                    yield percent

                else:
                    break

        cmd = "".join(['rtmpdump -r "', self.url, '" '])
        cmd_dl = "".join([cmd, '--flv "', self.target, '"'])
        cmd_resume = "".join([cmd, '--resume --flv "', self.target, '"'])
        cmd_resume_skip = "".join([cmd, '--resume --skip 1 --flv "',
                                                        self.target, '"'])
        SECONDS_TO_WAIT = 12
        max_skip_cnt = 10
        percent_re = re.compile("\((.+)%\)$")

        ret_code = None

        if some_dl_done or resuming:
            cmd = cmd_resume

        else:
            cmd = cmd_dl

        while need_more_dl:
            stderr_buff = ""
            whole_stderr_buff = ""

            if self.is_daemon:
                self.print_(u"Downloading ......\n {0}".format(cmd))

            logger.info(u"Fetch packets, command:\n{0}".format(cmd))
            p = subprocess.Popen(cmd, shell=True, stderr=subprocess.PIPE, 
                                                            close_fds=True)
            self.pid = p.pid + 1

            while ret_code is None:
                fds_read, fds_write, fds_stdn = select.select([p.stderr],
                                                    [], [], SECONDS_TO_WAIT)

                if len(fds_read) == 1:
                    c = p.stderr.read(1)
                    whole_stderr_buff += c

                    if c in ("\n","\r"):
                        match = percent_re.search(stderr_buff)

                        if match is not None:
                            max_skip_cnt = 10
                            yield float(match.group(1))

                        stderr_buff = ""

                    else:
                        stderr_buff += c

                ret_code = p.poll()

            whole_stderr_buff += p.stderr.read()
            self.pid = None

            if ret_code == 0:
                yield 100.0
                break

            elif ret_code == 2:
                cmd = cmd_resume

            else:
                must_resume = False
                for line in whole_stderr_buff.split("\n"):
                    if line.find("Couldn't resume FLV file, try --skip 1") != -1:
                        must_resume = True
                        break

                if must_resume and max_skip_cnt >= 0:
                    max_skip_cnt -= 1
                    cmd = cmd_resume_skip

                else:
                    yield -1.0

            ret_code = None

    def abort_subprocess(self):
        if self.pid is not None:
            try:
                os.kill(self.pid, signal.SIGINT)
            except:
                # Not sure if is still alive
                pass

    def remove_tempfile(self):
        try:
            os.unlink(self.target)
        except Exception as why:
            self.print_(u"Remove temp file {0}\n\tError: {1}"
                                    .format(self.target, why))

    def rename_target(self):
        count = 1
        dest = self.dest
        caput, cauda = os.path.splitext(dest)
        self.renamed = False

        while 1:
            if os.path.isfile(dest):
                dest = "".join([caput, '(', str(count), ')', cauda])
                count +=1
            else:
                break

        try:
            shutil.move(self.target, dest)
        except Exception as why:
            self.print_(u"Renaming error: \n\t\tfile: {0}\n\t    to: "\
                                "{1}\n\t    Reason: {2}"
                                .format(self.target, dest, why))

        else:
            self.print_(u"File: {0}\n\t  Renamed: {1} "
                                .format(self.target, dest))
            self.main.last_loaded = dest

    def print_(self, msg):
        """This print function is used in mode daemon only.

        """
        print u"{0}  {1}".format(time.time(), msg)


class LiveLoader(QObject):
    loadProgress = pyqtSignal(float)
    loadComplete = pyqtSignal(str)
    def __init__(self, main):
        # Parts of this code are come from TVDownloader
        # https://launchpad.net/~chaoswizard/+archive/tvdownloader
        super(LiveLoader, self).__init__()
        self.main = main
        self.is_loading = False
        self.kill_request = False
        self.is_daemon = False
        self.url = None
        self.target = None
        self.pattern = re.compile("(?P<scheme>[^:]*)://(?P<host>[^/^:]*):"
                                    "{0,1}(?P<port>[^/]*)/(?P<app>.*?)/"
                                    "(?P<playpath>\w*?\:.*)", re.DOTALL)

    def set_url(self, url):
        self.url = url

    def set_target(self, target):
        if self.is_daemon:
            # When python is invoked by cron the file system encoding
            # is not usable. Maybe related with this issue:
            # http://bugs.python.org/issue13643
            self.target = target.encode('utf-8', 'replace')

        else:
            self.target = target.encode(sys.getfilesystemencoding(), 'replace')

    def set_destination(self, dest):
        if self.is_daemon:
            self.dest = dest.encode('utf-8', 'replace')

        else:
            self.dest = dest.encode(sys.getfilesystemencoding(), 'replace')

    def format_command(self):
        match = re.match(self.pattern, self.url)
        cmd = ""
        if match != None:
            cmd = " ".join(['rtmpdump', '--host', '%host%', '--port', 
                            '%port%', '--protocol', '%scheme%', 
                            '--app %app%', '--playpath', '%playpath%'])
            cmd = cmd.replace('%scheme%', match.group('scheme'))\
                            .replace('%host%', match.group('host'))\
                            .replace('%app%', match.group('app'))\
                            .replace('%playpath%', match.group('playpath'))
            if( match.group('port') != ""):
                cmd = cmd.replace('%port%', match.group('port'))
            elif(self.url[ :6 ] == 'rtmpte'):
                cmd = cmd.replace('%port%', '80')
            elif(self.url[ :5 ] == 'rtmpe'):
                cmd = cmd.replace( '%port%', '1935')
            elif(self.url[ :5 ] == 'rtmps'):
                cmd = cmd.replace('%port%', '443')
            elif(self.url[ :5 ] == 'rtmpt'):
                cmd = cmd.replace('%port%', '80')
            else:
                cmd = cmd.replace('%port%', '1935')

        else:
            cmd = 'rtmpdump -r ' + self.url

        return cmd

    def load(self):
        if(self.url[ :4 ] == "rtmp"):
            cmd = self.format_command()
            cmd = "".join([cmd, " -o ", self.target])

        elif(self.url[:4] == "http" or self.url[:3] == "ftp" 
                                            or self.url[:3] == "mms"):
            cmd = "".join(["msdl -c ", self.url, " -o ", self.target])

        else:
            return

        arguments = cmd.split()
        if self.is_daemon:
            self.print_(u"Downloading ......\n {0}".format(arguments))

        try:
            reply = subprocess.Popen(arguments, universal_newlines=True,
                                    stdout=subprocess.PIPE, 
                                    stderr=subprocess.STDOUT)
            self.is_loading = True

        except (IOError, OSError) as exc:
            self.print_(u"\nSubprocess ERROR: {0}".format(exc))
            self.loadComplete.emit(exc)
            self.is_loading = False
            return

        percent = "0"
        duration = "0"
        err = ""
        while 1:
            text = reply.stdout.readline()[:-1]
            if type(text) != str or text == '' and reply.poll() != None:
                self.is_loading = False 
                break

            elif type(text) == str and len(text) > 6:
                if "ERROR:" in text:
                    err = text
                    break

                elif " sec (" in text:
                    chains = text.split(" ")
                    try:
                        duration = chains[3]
                    except:
                        pass

                    try:
                        percent = float(chains[5][1:-2])
                    except Exception as exc:
                        percent = 2

                    self.loadProgress.emit(percent)

            if self.kill_request:
                err = u"Loading aborted by user"
                break

        try:
            reply.kill()
        except:
            pass

        self.is_loading = False
        self.rename_target()
        self.loadComplete.emit(err)

    def rename_target(self):
        count = 1
        dest = self.dest
        caput, cauda = os.path.splitext(dest)
        while 1:
            if os.path.isfile(dest):
                dest = "".join([caput, '(', str(count), ')', cauda])
                count +=1
            else:
                break

        try:
            os.rename(self.target, dest)
        except Exception as why:
            self.print_(u"Renaming error: \n\t\t file: {0}\n\t    to: "\
                            "{1}\n\t    Reason: {2}"
                            .format(self.target, dest, why))

        else:
            self.print_(u"File: {0}\n\t   Renamed: {1} "
                            .format(self.target, dest))
            self.main.last_loaded = dest

    def print_(self, msg):
        print u"{0}  {1}".format(time.time(), msg)

class RateCounter(object):
    def __init__(self, site, fname):
        """Calculate and show the downloading rate.

        Args :
        site -- ArtePlus or ArteLiveWeb instance
        movie -- instance of movie
        """
        self.site = site
        lang = QLocale.system().name()
        self.locale = QLocale(lang)
        self.target = fname
        self.time_base = time.time()
        self.size = 0
        if os.path.isfile(self.target):
            inf = os.stat(self.target)
            self.size = inf.st_size

    def counter(self, p):
        """Evaluate the rate of downloading.

        Args :
        p -- percent

        Returns:
        Bandwidth, estimated remaining time
        """
        loop = 0
        size = 0
        while loop < 3:
            try:
                size = os.path.getsize(self.target)
            except OSError, why:
                loop += 1
                time.sleep(0.3)

            else:
                break

        if not size:
            return "", ""

        delta = size - self.size
        t =  time.time() - self.time_base
        r = int ((size / t) / 1024)
        rate = self.locale.toString(r)
        rem = ((t / p) * 100) - t
        m, s = divmod(rem, 60)
        st = QString(_("Speed:"))
        st2 = QString(_("Remaining time:"))
        rate_txt = "%s %s Kio/sec." % (st, rate)
        rem_txt = "%s %s min. %s sec." % (st2, int(m), int(s))

        return rate_txt, rem_txt


