# -*- coding: utf-8 -*-

# parsers.py
# This file is part of Qarte
#    
# Author : Vincent Vande Vyvre <vincent.vandevyvre@swing.be>
# Copyright: 2011-2013 Vincent Vande Vyvre
# Licence: GPL3
# Home page : https://launchpad.net/qarte
#
# Parser for pages arte+7 and arteLiveWeb

import os, re
import time
import urllib2, xml.dom.minidom
import sys
import pickle
import copy
import logging
logger = logging.getLogger(__name__)

from threading import Thread, current_thread
from threading import enumerate as enumerate_
from email.utils import parsedate

from PyQt4.QtCore import QObject, pyqtSignal

from data import CATEGORIES


class PlusParser(QObject):
    parsingFinished = pyqtSignal(list)
    updatingFinished = pyqtSignal()
    ARTE_WEB_ROOT = 'http://videos.arte.tv'
    RE_TITLE = re.compile('(?<=<h2><a href=")(.*?)(?=</a></h2>)', re.DOTALL)
    RE_PITCH = re.compile('(?<=<p class="teaserText">)(.*?)(?=</p>)', re.DOTALL)
    RE_DURATION = re.compile('(?<=<div class="duration_thumbnail">)'
                                        '(.*?)(?=</div>)', re.DOTALL)
    RE_DATE = re.compile('(?<=<p>)(.*?)(?=</p>)', re.DOTALL)
    RE_PICT = re.compile('(?<=class="thumbnail")(.*?)(?=/></a>)', re.DOTALL)
    RE_TIME = re.compile("^\d\d[h:]\d\d$")
    RE_XML = re.compile('(?<=vars_player.videorefFileUrl)(.*?)(?=";)', re.DOTALL)
    RE_XML2 = re.compile('ajaxUrl:.?.(/../do_delegate/videos/)'
                                        '([^,]+),view,commentForm.html')
    RE_STREAM = re.compile('<url quality="([^"]+)">([^<]+)<')
    RE_CONT = re.compile('(?<="recentTracksCont">)(.*?)(?="technicalSheet">)', 
                                        re.DOTALL)
    RE_DE = re.compile('(?<=<p>)(.*?)(?=</p>)', re.DOTALL)
    RE_OR = re.compile('(?<=<p style="margin-top: 0">)(.*?)(?=<br/>)', 
                                        re.DOTALL)
    MONTHS = [u"ja", u"f", u"mar", u"av", u"mai", u"juin", u"juil", 
                       u"ao", u"se", u"oc", u"no", u"d", u"Ja", 
                       u"F", u"Mä", u"Ap", u"Mai", u"Jun", u"Jul", 
                       u"Au", u"Se", u"Ok", u"No", u"De"]

    def __init__(self, main, lang, videos):
        """Instanciate the arte+7 parser.

        Args:
        main -- ArtePlus instance
        lang -- 'fr' or 'de'
        video -- reference of main's video list (empty)
        """
        super(PlusParser, self).__init__()
        self.main = main
        self.videos = videos
        self.lang = lang
        self.page = "/".join([self.ARTE_WEB_ROOT, lang, "videos/"])
        self.fdesc = os.path.join(main.user_fld, 'videoDesc')
        self.wglog = os.path.join(main.user_fld, 'wgetLog')
        self.findex = os.path.join(main.user_fld, 'summaries')
        self.is_updating = False
        self.close_app = False
        self.is_daemon = False

    def parse(self):
        """Get arte+7 page and run parsing.

        """
        # Stage 1
        logger.info(u"Run arte+7 parser: {0}".format(current_thread()))
        logger.info(u"Get page: {0}".format(self.page))
        content, err = self.get_page_content(self.page)

        if content is None:
            logger.info(u"Read Error: {0}".format(err))
            self.parsingFinished.emit(['error', 'Stage 1', err])
            return

        if not len(content):
            logger.info(u"Read Error: 'Page empty'")
            self.parsingFinished.emit(['error', 'Stage 1', 'Page empty'])
            return

        # Stage 2
        # Here, we need to find one <script> wich contains "videowallSettings"
        # into this part, we seek for a string like:
        #   "/fr/do_delegate/videos/index-1234567,view,asThumbnail.html"
        # finally, we use this string to create our url
        #   "http://videos.arte.tv/fr/do_delegate/videos/index-1234567,
        #    view,asThumbnail.html,?hash=fr/thumb///1/200/"
        # Note: the last number 200 is the maximum videos to display.

        chs = content.split('videowallSettings = {')[1]\
                            .split('coverflowXmlUrl')[0]
        found_url = 0

        for chain in chs.split():
            if "asThumbnail" in chain:
                page_url = "".join([self.ARTE_WEB_ROOT,
                                    chain.replace('"',''), "?hash=",
                                    self.lang, "/thumb///1/200/"])
                found_url = 1
                break

        if not found_url:
            logger.info(u"Read Error: 'URL videowall not found'")
            self.parsingFinished.emit(['error', 'Stage 2', 'URL not found'])
            return

        # Stage 3
        # Here we separate the page in blocks 
        #   '<div class="video">
        #       ...
        #   </div>'
        logger.info(u"Get page: {0}".format(page_url))
        content, err = self.get_page_content(page_url)

        if content is None:
            logger.info(u"Read Error: {0}".format(err))
            self.parsingFinished.emit(['error', 'Stage 3', str(err)])
            return

        # Parsing for each video
        items = content.split('<div class="video">')[1:]
        self.lang = "/{0}/".format(self.lang)

        for it in items:
            vid = self.get_videos_data(it)

            if vid is not None:
                video = VideoPlusItem(vid)
                self.videos.append(video)

        logger.info(u"Found {0} videos".format(len(self.videos)))
        self.parsingFinished.emit([None])

    def get_videos_data(self, txt):
        """Extract the video properties with the help of regex.

        Args:
        txt -- block '<div class="video"> .... </div>'
        """
        try:
            ttl = self.RE_TITLE.search(txt).group(0)
            ref, title = ttl.split('html">')
            title = title.decode('utf-8', 'replace')
            title = self.html_to_utf8(title)
            ref = self.ARTE_WEB_ROOT + ref
            link = ref.replace("/fr/", self.lang)
        except Exception, why:
            logger.info('video properties error: {0}'.format(why))
            return None

        try:
            dt = self.RE_DATE.search(txt).group(0)
            date = dt.decode('utf-8', 'replace')
            date = self.format_date(date)
        except Exception, why:
            logger.info('date error for {0}: {1}'.format(title, why))
            date = None

        try:
            duration = self.RE_DURATION.search(txt).group(0)
        except Exception, why:
            logger.info('duration error for {0}: {1}'.format(title, why))
            duration = 'Unknow'

        try:
            pict = self.RE_PICT.search(txt).group(0)
            pict = pict.split('src="')[1].strip()[:-1]
            pict = self.ARTE_WEB_ROOT + pict
            pict = pict.replace("/fr/", self.lang)
        except Exception, why:
            logger.info('thumbnail error for {0}: {1}'.format(title, why))
            pict = None

        try:
            ptc = self.RE_PITCH.search(txt).group(0)
            ptc = ptc.decode('utf-8', 'replace')
            pitch = self.html_to_utf8(ptc)
        except Exception, why:
            logger.info('pitch error for {0}: {1}'.format(title, why))
            pitch = None

        if link[-1] == '.':
            link = link[:-1]

        return [title, date, duration, link, pict, pitch]

    def html_to_utf8(self, text):
        return text.replace("&#39;", "'")

    def format_date(self, dt):
        """Normalise the date.

        Args:
        dt -- str(date) wich may have various formats;
                'Aujourd'hui, 00h15', 'Hier, 23h14', 'Dim., 26 août 2012, 15h32'

        Returns:
        str(%Y %m %d %Hh%M)
        """
        date_array = dt.split(",")

        if self.RE_TIME.search(date_array[-1].strip()) is None:
            return ""

        time_ = date_array[-1].strip()

        if date_array[0].strip() in (u"Aujourd'hui", u"Heute"):
            date_ = time.strftime("%Y %m %d", time.localtime(time.time()))

        elif date_array[0].strip() in (u"Hier", u"Gestern"):
            date_ = time.strftime("%Y %m %d", 
                                    time.localtime(time.time() - (86400)))

        else:
            array = date_array[1].split()
            day = array[0].strip(".")
            month = array[1]

            for idx, mnt in enumerate(self.MONTHS):
                if month.startswith(mnt):
                    if idx > 11:
                        idx -= 12
                    month = "%02d" % (idx+1)
                    break

            year = array[2]
            date_ = "%s %s %s" % (year, month, day)

        return date_ + ", " + time_

    def get_page_content(self, url):
        """Fetch a html or xml page.

        Args:
        url -- page URL

        Returns:
        page as text or None if fail
        """
        try:
            content = urllib2.urlopen(url).read()
        except IOError as why:
            return None, why

        return content, None

    def update_all(self):
        """Called after the viewer is completed for update the video infos.

        """
        self.is_updating = True
        Thread(target=self.update_videos_infos, args=(None,)).start()

    def update_videos_infos(self, *args):
        """Update the summary with file 'summaries'.

        """
        logger.info(u"Update arte+7 summaries list")
        logger.info(u"Threads enum: {0}".format(enumerate_()))
        begin = time.time()
        self.load_summaries()

        if not self.summaries:
            # First use, we need to create the file 'summaries'
            self.get_new_infos()
            return

        for idx, video in enumerate(self.videos):
            # summaries is a dict where the keys are the video's datetime 
            if video.date in self.summaries:
                video.summary = self.summaries[video.date][0]
                # orig is '(country, anno)'
                video.orig = self.summaries[video.date][1]

        self.get_new_infos()

    def get_new_infos(self):
        """Create the file summaries at first use.

        """
        for video in self.videos:
            if self.close_app:
                # exit requested by the user
                break

            if video.summary is None:
                infos = self.fetch_summary(video.link)
                video.summary, video.orig = infos[0], infos[1]
                self.summaries[video.date] = infos

        logger.info(u"arte+7 summaries updated")
        self.main.summaries = self.summaries
        self.save_videos_infos()
        self.updatingFinished.emit()
        self.is_updating = False

    def fetch_summary(self, link):
        """Fetch the video page and extract the description and origin.

        Args:
        link -- page URL
        """
        reply = self.get_page_content(link)

        if reply[0] is None:
            return None, None

        content = reply[0]
        try:
            txt = self.RE_CONT.search(content).group(0)
        except Exception, why:
            return None, u""
        
        try:
            d = self.RE_DE.search(txt).group(0)
            desc = d.decode('utf-8', 'replace')
        except Exception, why:
            desc =u"No description"

        try:
            o = self.RE_OR.search(txt).group(0)
            orig = o.decode('utf-8', 'replace')
        except Exception, why:
            orig =u""

        return desc, orig

    def fetch_stream_links(self, link):
        """Fetch the XML page of the video and extract the stream URL.

        args:
        link -- video page URL
        """
        urls = []
        logger.info(u"Get stream URL: {0}".format(link))
        reply = self.get_page_content(link)

        if reply[0] is None:
            logger.warning(u"Read error: {0}".format(reply[1]))
            return None, None

        content = reply[0]
        #RE_XML: ('(?<=vars_player.videorefFileUrl)(.*?)(?=";)', re.DOTALL)
        match = self.RE_XML.search(content)

        if match is not None:
            st = match.group(0)
            # Space and other symbol may vary
            LNK = re.compile('(?<=http)(.*?)(?=xml)', re.DOTALL)
            st = LNK.search(match.group(0))

            if st is not None:
                xmlurl = "".join(['http', st.group(0), 'xml'])

            logger.info(u"Found: {0}".format(xmlurl))

        else:
            #RE_XML2: ('ajaxUrl:.?.(/../do_delegate/videos/)'
                                        #'([^,]+),view,commentForm.html')
            match = self.RE_XML2.search(content)
            if match is not None:
                xmlurl = 'http://videos.arte.tv%s%s,view,asPlayerXml.xml'\
                                        % (match.group(1), match.group(2))
            else:
                logger.warning(u"Unable to find stream URl")
                return None, None

        # Now we have the xml URL
        xmlurl = xmlurl.replace("'", "")
        logger.info(u"Get xml page: {0}".format(xmlurl))
        reply = self.get_page_content(xmlurl)

        if reply[0] is None:
            logger.warning(u"Read error: {0}".format(reply[1]))
            return None, None

        content = reply[0].split("\n")
        lg = "".join(['<video lang="', self.lang[1:-1], '" ref="'])
        lnk = ""
        for line in content:
            if line.startswith(lg):
                lnk = line[22:].split(" ")[0][:-1]
                break

        if not lnk:
            logger.warning(u"Unable to find stream URl")
            return None, None

        reply = self.get_page_content(lnk)
        if reply[0] is None:
            logger.warning(u"Read error: {0}".format(reply[1]))
            return None, None

        content = reply[0].split("\n")
        for line in content:
            # regex: '<url quality="([^"]+)">([^<]+)<'
            match = self.RE_STREAM.search(line)
            if match:
                urls.append((match.group(1), match.group(2)))

        if not urls:
            logger.warning(u"rtmp URLs not found in: {0}".format(link))
            return (None, None)

        hd, sd = None, None
        for u in urls:
            if u[0] == 'hd':
                hd = u[1]

            elif u[0] == 'sd':
                sd = u[1]

        return hd, sd

    def save_content(self, txt):
        """debugging function"""
        with open("contentxml", "w") as objf:
            objf.write(txt)

    def load_summaries(self):
        """Summaries are saved in a dictionnary where keys are the video's dates.

        """
        try:
            with open(self.findex, "r") as objfile:
                self.summaries = pickle.load(objfile)
        except:
            # Assume is first use
            self.summaries = {}

    def save_videos_infos(self):
        try:
            with open(self.findex, 'w') as objf:
                pickle.dump(self.summaries, objf)
        except Exception as why:
            logger.warning(u"Unable to save summaries list, reason: \n\t{0}"
                                .format(why))


class LiveParser(QObject):
    loadingCategoryFinished = pyqtSignal(str)
    loadingCategoryFailed = pyqtSignal(str)
    def __init__(self, alw, URLS_FR, URLS_DE):
        """Instanciate the arteLiveWeb parser.

        Args:
        alw -- ArteLiveWeb instance
        """
        super(LiveParser, self).__init__()
        self.alw = alw
        self.lang = alw.lang
        ARTE_WEB_ROOT = 'http://liveweb.arte.tv/{0}'.format(self.lang)
        cat_re = re.compile("".join(['<li><a href="http://liveweb.arte.tv/',
                                    self.lang,
                                    '/cat/(?P<lien>.*?)" class="accueil"',
                                    '>(?P<nom>.*?)</a></li>']), re.DOTALL)
        count = 0
        self.categories = {}

        self.urls = URLS_FR
        if self.lang == 'de':
            self.urls = URLS_DE

        # Stage 1
        logger.info(u"Get page: {0}".format(ARTE_WEB_ROOT))

        reply = self.get_page(ARTE_WEB_ROOT)
        if reply[0] is None:
            self.alw.on_page_failed("Stage 1", reply[1])
            return

        # Stage 2
        for item in re.finditer(cat_re, reply[0]):
            self.categories[item.group('nom').decode('utf-8', 'replace')] \
                                                    = item.group('lien')

        self.categories[u'La Blogothèque'] = 'blogo'
        self.alw.set_categories(self.categories)

    def get_videos_list(self, cat):
        # Stage 3
        logger.info(u"Get arteLiveWeb video's data in '{0}'".format(cat))
        logger.info(u"Threads enum: {0}".format(enumerate_()))

        re_items = re.compile("(<item>.*?</item>)", re.DOTALL)
        #videos = {}
        #value = self.categories[cat]
        #url = "http://liveweb.arte.tv/{0}/cat/{1}".format(self.lang, value)
        urls = self.urls[cat]
        vids = {}
        for url in urls:
            page = self.get_page(url)

            if page[0] is None:
                self.loadingCategoryFailed.emit(str(page[1]))
                continue

            if len(page[0]) > 0:
                items = re.findall(re_items, page[0])
                #videos[cat] = items

            else:
                self.loadingCategoryFailed.emit('page is empty')
                continue


            # Stage 4
            for item in items:
                it = self.get_element(item)

                if it is None:
                    continue

                if it['ID'] in vids:
                    # Duplicate
                    continue

                try:
                    rfc_date = parsedate(it['date'])
                    lt = time.localtime(time.mktime(rfc_date))
                    it['date'] = time.strftime('%c', lt).decode('utf-8', 'ignore')
                    if time.mktime(rfc_date) - float(time.time()) > 0:
                        it['soon'] = True
                except:
                    it['date'] = 'Date ?'

                itm = VideoItem(cat, it)
                vids[it['ID']] = itm

        logger.info('category {0} has {1} videos'.format(cat, len(vids)))
        self.alw.videos[cat] = vids
        self.loadingCategoryFinished.emit(CATEGORIES[self.lang][cat])

    def get_element(self, chain):
        ele = {'HD': None, 'SD': None, 'Live': None, 'soon': False}
        count = 0
        tt = re.compile("(?<=<title>)(.*?)(?=</title>)", re.DOTALL)
        lk = re.compile("(?<=<link>)(http://liveweb.arte.tv/{0}/video/.*?)"
                        "(?=</link>)".format(self.lang), re.DOTALL)
        dt = re.compile("(?<=<pubDate>)(.*?)(?=</pubDate>)", re.DOTALL)
        pt = re.compile("(?<=<description>)(.*?)(?=</description>)", re.DOTALL)
        at = re.compile("(?<=<author>)(.*?)(?=</author>)", re.DOTALL)
        en = re.compile("<enclosure.*?/event/.*?/(.*?)-.*?/>", re.DOTALL)
        pix = re.compile('(?<=<enclosure url=")(.*?)(?=" type="image/)')
      
        try:
            s = tt.search(chain).group(0)
            ele['title'] = s.decode('utf-8', 'replace')
        except:
            return None

        try:
            ele['link']  = lk.search(chain).group(0)
        except Exception, why:
            ele['link']  = "No link"

        try:
            s = (dt.search(chain).group(0))
            ele['date'] = s.decode('utf-8', 'replace')
        except:
            ele['date'] = "No date"

        try:
            s = (pt.search(chain).group(0))
            ele['pitch'] = s.decode('utf-8', 'replace')
        except:
            ele['pitch'] = "No description"

        try:
            s = (at.search(chain).group(0))
            ele['author'] = s.decode('utf-8', 'replace')
        except:
            ele['author'] = "Unknow"

        try:
            ele['ID'] = int(en.search(chain).group(1))
        except:
            ele['ID'] = 0

        try:
            ele['pict'] = pix.search(chain).group(0)
        except:
            ele['pict'] = None

        return ele

    def get_page(self, url):
        logger.info('get page {0}'.format(url))
        try:
            content = urllib2.urlopen(url).read()
        except IOError as why:
            return None, why

        return self.unescape(content), None

    def get_links(self, idx):
        page = self.get_page("http://arte.vo.llnwd.net/o21/liveweb/events/event-{0}.xml".format(idx))

        if page[0] is None:
            self.alw.on_page_failed("Get stream's URL", page[1])
            return

        page = page[0]
        HD = re.compile("(?<=<urlHd>)(.*?)(?=</urlHd>)", re.DOTALL)
        SD = re.compile("(?<=<urlSd>)(.*?)(?=</urlSd>)", re.DOTALL)
        Live = re.compile("(?<=<liveUrl>)(.*?)(?=</liveUrl>)", re.DOTALL)
        obj = {}

        try:
            obj['HD'] = HD.search(page).group(0)
        except AttributeError:
            obj['HD'] = None

        try:
            obj['SD'] = SD.search(page).group(0)
        except AttributeError:
            obj['SD'] = None

        try:
            obj['Live'] = Live.search(page).group(0)
        except AttributeError:
            obj['Live'] = None

        return obj

    def unescape(self, text):
        def ent2chr( m ):
            code = m.group( 1 )

            if( code.isdigit() ): 
                code = int( code )

            else:
                code = int( code[ 1 : ], 16 )

            if( code < 256 ): 
                return chr( code )

            else: 
                return '?'

        text = text.replace("&lt;", "<")
        text = text.replace("&gt;", ">")
        text = text.replace("&amp;", "&")

        return text

class ExtraParser(QObject):
    loadingCategoryFinished = pyqtSignal(str)
    loadingCategoryFailed = pyqtSignal(str)
    ITEM = 'http://arte.vo.llnwd.net/o21/liveweb/events/event-[idx].xml'
    RE_ITEMS = re.compile('(?<=<item>)(.*?)(?=</item>)', re.DOTALL)
    RE_HD = re.compile('(?<=<urlHd>)(.*?)(?=</urlHd>)', re.DOTALL)
    TITLE = re.compile("(?<=<title>)(.*?)(?=</title>)", re.DOTALL)
    DATE = re.compile("(?<=<pubDate>)(.*?)(?=</pubDate>)", re.DOTALL)
    DESC = re.compile("(?<=<description>)(.*?)(?=</description>)", re.DOTALL)
    AUTHOR = re.compile("(?<=<author>)(.*?)(?=</author>)", re.DOTALL)
    EVENT = re.compile("<enclosure.*?/event/.*?/(.*?)-.*?/>", re.DOTALL)
    IMG = re.compile('(?<=<enclosure url=")(.*?)(?=" type="image/)')

    def __init__(self, aext):
        """Instanciate the arteLiveWeb parser.

        Args:
        alw -- ArteLiveWeb instance
        """
        super(ExtraParser, self).__init__()
        self.aext = aext
        self.lang = aext.lang
        self.partners = aext.partners
        self.LINK = re.compile("(?<=<link>)(http://liveweb.arte.tv/{0}/video/.*?)"
                                "(?=</link>)".format(self.lang), re.DOTALL)

    def read_partner(self, part):
        content = self.get_page(self.partners[part].replace('§', self.lang))
        if content[0] is None:
            logger.info('content of {0} is None: {1}'.format(part, content[1]))
            return

        items = self.RE_ITEMS.findall(content[0])
        vids = {}

        for it in items:

            vid = self.get_element(it)
            if vid is None:
                continue

            try:
                rfc_date = parsedate(vid['date'])
                lt = time.localtime(time.mktime(rfc_date))
                vid['date'] = time.strftime('%c', lt).decode('utf-8', 'ignore')
                if time.mktime(rfc_date) - float(time.time()) > 0:
                    vid['soon'] = True
            except:
                vid['date'] = 'Date ?'

            itm = VideoItem(part, vid)
            vids[vid['ID']] = itm

        self.aext.videos[part] = vids
        self.loadingCategoryFinished.emit(part)

    def get_page(self, url):
        logger.info('get page {0}'.format(url))
        try:
            content = urllib2.urlopen(url).read()
        except IOError as why:
            logger.info('page error: {0}'.format(why))
            return None, why

        return self.unescape(content), None

    def get_links(self, idx):
        page = self.get_page("http://arte.vo.llnwd.net/o21/liveweb/events/event-{0}.xml".format(idx))

        if page[0] is None:
            return

        page = page[0]
        HD = re.compile("(?<=<urlHd>)(.*?)(?=</urlHd>)", re.DOTALL)
        SD = re.compile("(?<=<urlSd>)(.*?)(?=</urlSd>)", re.DOTALL)
        Live = re.compile("(?<=<liveUrl>)(.*?)(?=</liveUrl>)", re.DOTALL)
        obj = {}

        try:
            obj['HD'] = HD.search(page).group(0)
        except AttributeError:
            obj['HD'] = None

        try:
            obj['SD'] = SD.search(page).group(0)
        except AttributeError:
            obj['SD'] = None

        try:
            obj['Live'] = Live.search(page).group(0)
        except AttributeError:
            obj['Live'] = None

        return obj

    def unescape(self, text):
        def ent2chr( m ):
            code = m.group( 1 )

            if( code.isdigit() ): 
                code = int( code )

            else:
                code = int( code[ 1 : ], 16 )

            if( code < 256 ): 
                return chr( code )

            else: 
                return '?'

        text = text.replace("&lt;", "<")
        text = text.replace("&gt;", ">")
        text = text.replace("&amp;", "&")

        return text

    def get_element(self, chain):
        lang = 'fr'
        ele = {'HD': None, 'SD': None, 'Live': None, 'soon': False}
        count = 0

        try:
            s = self.TITLE.search(chain).group(0)
            ele['title'] = s.decode('utf-8', 'replace')
        except:
            return None

        try:
            ele['link']  = self.LINK.search(chain).group(0)
        except Exception, why:
            ele['link']  = "No link"

        try:
            s = self.DATE.search(chain).group(0)
            ele['date'] = s.decode('utf-8', 'replace')
        except:
            ele['date'] = "No date"

        try:
            s = (self.DESC.search(chain).group(0))
            ele['pitch'] = s.decode('utf-8', 'replace')
        except:
            ele['pitch'] = "No description"

        try:
            s = (self.AUTHOR.search(chain).group(0))
            ele['author'] = s.decode('utf-8', 'replace')
        except:
            ele['author'] = "Unknow"

        try:
            ele['ID'] = int(self.EVENT.search(chain).group(1))
        except:
            ele['ID'] = 0

        try:
            ele['pict'] = self.IMG.search(chain).group(0)
        except:
            ele['pict'] = None

        return ele


class VideoPlusItem(object):
    def __init__(self, data):
        """Item video in arte+7.

        Args:
        data -- values provided by the parser
        """
        d = dict(title = data[0],
                date = data[1],
                duration = data[2],
                link = data[3],
                pix = data[4],
                pitch = data[5],
                HD = None,
                SD = None,
                outfile = None,
                preview = None,
                summary = None,
                orig = None)

        for key, value in d.iteritems():
            setattr(self, key, value)


class VideoItem(object):
    def __init__(self, cat, item):
        """Item video in arteLiveWeb.

        Args:
        cat -- category
        item -- values provided by the parser
        """
        d = dict(category = cat,
                order = item['ID'],
                title = item['title'],
                date = item['date'],
                pitch = item['pitch'],
                link = item['link'],
                pix = item['pict'],
                soon = item['soon'],
                preview = None,
                HD = None,
                SD = None,
                Live = None,
                outfile = None,
                quality = None,
                author = item['author'])

        for key, value in d.iteritems():
            setattr(self, key, value)

        if self.pix is not None:
            self.preview = os.path.basename(self.pix)

    def get_stream_links(self, page):
        tt = re.compile("(?<=<nameDe>)(.*?)(?=</nameDe>)", re.DOTALL)
        HD = re.compile("(?<=<urlHd>)(.*?)(?=</urlHd>)", re.DOTALL)
        SD = re.compile("(?<=<urlSd>)(.*?)(?=</urlSd>)", re.DOTALL)
        Live = re.compile("(?<=<liveUrl>)(.*?)(?=</liveUrl>)", re.DOTALL)

        try:
            self.local_name = tt.search(page).group(0)
        except AttributeError:
            self.local_name = None

        try:
            self.HD = HD.search(page).group(0)
        except AttributeError:
            self.HD = None

        try:
            self.SD = SD.search(page).group(0)
        except AttributeError:
            self.SD = None

        try:
            self.Live = Live.search(page).group(0)
        except AttributeError:
            self.Live = None

    def get_image(self):
        if self.preview is not None:
            try:
                img = urllib2.urlopen(self.pix).read()
            except Exception, why:
                return None

            else:
                return img

        else:
            return None

